# Generated by roxygen2: do not edit by hand

S3method(eq5d,data.frame)
S3method(eq5d,default)
S3method(eq5d,matrix)
S3method(eq5dcf,data.frame)
S3method(eq5dcf,default)
S3method(eq5dcf,matrix)
S3method(lfs,data.frame)
S3method(lfs,default)
S3method(lfs,matrix)
S3method(lss,data.frame)
S3method(lss,default)
S3method(lss,matrix)
export(CW)
export(DSU3L)
export(DSU5L)
export(RCW)
export(TTO)
export(VAS)
export(VT)
export(Y)
export(eq5d)
export(eq5d3l)
export(eq5d5l)
export(eq5dcf)
export(eq5dcw)
export(eq5dds)
export(eq5dmap)
export(eq5drcw)
export(eq5dy)
export(getDimensionsFromHealthStates)
export(getHealthStates)
export(getHealthStatesFromDimensions)
export(get_all_health_states)
export(get_dimensions_from_health_states)
export(get_health_states_from_dimensions)
export(hpg)
export(hsdi)
export(lfs)
export(lss)
export(pchc)
export(ps)
export(shannon)
export(shiny_eq5d)
export(splitHealthStates)
export(valuesets)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(stats,weighted.mean)
